/* ************************************************************************** */
/*                                                                            */
/*                                                        :::      ::::::::   */
/*   main.c                                             :+:      :+:    :+:   */
/*                                                    +:+ +:+         +:+     */
/*   By: cchauvet <cchauvet@student.42angoulem      +#+  +:+       +#+        */
/*                                                +#+#+#+#+#+   +#+           */
/*   Created: 2023/02/16 15:16:14 by cchauvet          #+#    #+#             */
/*   Updated: 2023/02/24 11:04:40 by cchauvet         ###   ########.fr       */
/*                                                                            */
/* ************************************************************************** */

#include "libftx/libftx.h"
#include "minishell.h"
#include <readline/readline.h>

static char	*ft_get_user_input()
{
	char	*line;
	char	*prompt;
	char	*pwd;

	pwd = get_pwd(2);
	if (pwd == NULL)
		return (NULL);
	prompt = ft_strmerger(2, pwd, "$ ");
	free(pwd);
	if (prompt == NULL)
	{
		ft_eprintf("minishell: malloc failed\n");
		return (NULL);
	}
	line = readline(prompt);
	if (line != NULL && ft_strcmp(line, "") != 0)
		add_history(line);
	free(prompt);
	return (line);
}

static int	ft_minishell(t_data *data, char *line)
{
	t_list	**cmds;
	char	*line_clean;
	int		infile;
	int		outfile;
	int		return_value;

	if (ft_syntatic_verif(data, line))
		return (1);
	line_clean = ft_normalizer(line);
	if (line_clean == NULL)
		return (1);
	outfile = ft_outfile(data, line_clean);
	return_value = 0;
	if (outfile == -2)
		return_value = 1;
	infile = ft_infile(data, line_clean);
	if (return_value == 0 && infile == -2)
		return_value = 1;
	if (return_value == 0&& ft_gen_exit_code_var(data))
		return_value = 1;
	cmds = ft_parse_cmds(data, line_clean, infile, outfile);
	if (return_value == 0 && cmds == NULL)
		return_value = 1;
	if (return_value == 0 && ft_cmds_executor(data, cmds))
		return_value = 1;
	if (outfile > 2)
		close(outfile);
	if (infile > 2)
		close(infile);
	ft_lstclear(cmds, ft_cmddel);
	free(cmds);
	free(line_clean);
	return (-1 * return_value);
}

void	ft_ctrlc(int num)
{
	(void) num;
	if (*ft_get_heredoc())
	{
		kill(*ft_get_heredoc(), SIGQUIT);
	}
	else
	{
		rl_replace_line("", 0);
		rl_on_new_line();
		ft_putchar_fd('\n', 1);
		rl_redisplay();
	}
}

void	ft_quit(int num)
{
	(void) num;
	if (*ft_get_heredoc())
	{
		ft_printf("pb");
	}
	else
		ft_printf("bozoman");
}

int	main(int ac, char **av, char **env)
{
	t_data	data;
	char	*line;

	(void) ac;
	(void) av;
	signal(SIGINT, ft_ctrlc);
	signal(SIGQUIT, ft_quit);
	data.exit_code = 0;
	data.env = init_env(env);
	if (data.env == NULL)
		return (1);
	line = ft_get_user_input();
	while (line != NULL)
	{
		if (ft_minishell(&data, line) == -1)
			break ;
		free(line);
		line = ft_get_user_input();
		if (line == NULL)
			break ;
	}
	ft_lstclear(data.env, env_del);
	free(data.exit_code_str);
	free(data.env);
	return (data.exit_code);
}
